@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.2.0 (_PROJECT_BUILD_ID_)
+####  Version 2.3.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -96,7 +96,7 @@ find.interaction.rfsrc <- function(
                                         "permute.ensemble", "random.ensemble", "anti.ensemble"))
   event.info <- get.event.info(object)
   n.event <- max(1, length(event.info$event.type))
-  outcome.target <- coerce.multivariate.target(object, outcome.target)
+  outcome.target <- get.univariate.target(object, outcome.target)
   if (n.event > 1) {
     object.imp <- NULL
     interact.imp.list.names <- paste("event.", 1:length(event.info$event.type), sep = "")
