@@ -2,7 +2,7 @@
 ####**********************************************************************
 ####
 ####  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-####  Version 2.2.0 (_PROJECT_BUILD_ID_)
+####  Version 2.3.0 (_PROJECT_BUILD_ID_)
 ####
 ####  Copyright 2016, University of Miami
 ####
@@ -64,7 +64,7 @@ predict.rfsrc <-
   function(object,
            newdata,
            outcome.target=NULL,
-           importance = c(FALSE, TRUE, "none", "permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble")[4],
+           importance = c(FALSE, TRUE, "none", "permute", "random", "anti", "permute.ensemble", "random.ensemble", "anti.ensemble")[1],
            na.action = c("na.omit", "na.impute"),
            outcome = c("train", "test"),
            proximity = FALSE,
@@ -72,7 +72,7 @@ predict.rfsrc <-
            split.depth = c(FALSE, "all.trees", "by.tree"),
            seed = NULL,
            do.trace = FALSE,
-           membership = TRUE,
+           membership = FALSE,
            statistics = FALSE,
            ...)
 {
